*******************************************************************************
*  Sample Conversion Definition File (CDF) for TEXT2DBF                       *
*                                                                             *
*    - Blank lines and lines beginning with "*" are ignored                   *
*                                                                             *
*    - Each line defines a field in the destination database and an           *
*      expression that retrieves the data from the source file.  The          *
*      expression may can contain any Clipper 5 function to convert or        *
*      manipulate the source data.  Two additional user-defined functions     *
*      are included and may be used: VALIMPDEC() - converts a numeric string  *
*      with an implied decimal to Clipper numeric, CDT2DATE() - converts a    *
*      string date in many formats to a Clipper date.                         *
*                                                                             *
*    - 2 variables are available for expressions:                             *
*         nCnt  - text record counter                                         *
*         cRec  - entire text record                                          *
*                                                                             *
*    - Field definition format is as follows:                                 *
*         field_name | type | length | decimals | expression                  *
*                                                                             *
*      The pipes (|) are REQUIRED item separators.  All items are required    *
*      and must abide by Clipper 5 field definition rules.                    *
*                                                                             *
*    - Sample expressions:                                                    *
*        For the following examples, assume the record counter nCnt = 12      *
*        and the record cRec contains the following:                          *
*           081292My dog has fleas 100 1FOO                                   *
*                                                                             *
*        CDT2DATE(SUBSTR(cRec,1,6),"M2D2Y2")    ==> 08/12/92           (date) *
*        CDT2DATE(SUBSTR(cRec,1,6),"D2M2Y2")    ==> 12/08/92           (date) *
*        SUBSTR(cRec,7,16)                      ==> My dog has fleas   (char) *
*        VAL(SUBSTR(cRec,24,3))                 ==>        100         (num)  *
*        IIF(SUBSTR(cRec,28,1)=="0",.F.,.T.)    ==> .T.                (log)  *
*        SUBSTR(cRec,29,3)+"BAR"                ==> FOOBAR             (char) *
*        "X"                                    ==> X                  (char) *
*        nCnt                                   ==>         12         (num)  *
*        STR(nCnt,6)                            ==>     12             (char) *
*        VALIMPDEC(SUBSTR(cRec,1,6),2)          ==>     812.92         (num)  *
*        VALIMPDEC(SUBSTR(cRec,1,6),4)          ==>     8.1292         (num)  *
*                                                                             *
*******************************************************************************

ACCOUNT    |C|  7|  0| SUBSTR(cRec,1,7)
DATE       |D|  8|  0| CDATE2DT(SUBSTR(cRec,10,6))
P_NUMBER   |C|  7|  0| SUBSTR(cRec,16,7)
P_DATE     |D|  8|  0| CDATE2DT(SUBSTR(cRec,25,6))
P_TYPE     |C|  1|  0| SUBSTR(cRec,31,1)
P_MAP      |C|  5|  0| SUBSTR(cRec,32,5)
P_BLOCK    |C|  5|  0| SUBSTR(cRec,37,5)
P_LOT      |C|  4|  0| SUBSTR(cRec,42,4)
P_INDEX    |C|  7|  0| SUBSTR(cRec,47,7)
OWNER      |C| 35|  0| SUBSTR(cRec,54,35)
ADDR1      |C| 35|  0| SUBSTR(cRec,89,35)
ADDR2      |C| 35|  0| SUBSTR(cRec,124,35)
ADDR3      |C| 35|  0| SUBSTR(cRec,159,35)
ADDR4      |C| 35|  0| SUBSTR(cRec,194,35)
STATE      |C|  2|  0| SUBSTR(cRec,229,2)
ZIP        |C|  9|  0| SUBSTR(cRec,232,5)+SUBSTR(cRec,239,4)
COUNTRY    |C|  1|  0| SUBSTR(cRec,243,1)
HISTORY1   |C|  7|  0| SUBSTR(cRec,245,7)
HISTORY2   |C|  7|  0| SUBSTR(cRec,253,7)
ACREAGE    |N|  8|  2| VALIMPDEC(SUBSTR(cRec,261,7))
REV_STAMPS |N| 12|  2| VALIMPDEC(SUBSTR(cRec,269,11))
DEF_CODE   |C|  1|  0| SUBSTR(cRec,280,1)
DEF_AMOUNT |N|  9|  0| VAL(SUBSTR(cRec,282,9))
PROP_DESC  |C| 40|  0| SUBSTR(cRec,291,40)
D_BOOK     |C|  4|  0| SUBSTR(cRec,331,4)
D_PAGE     |C|  4|  0| SUBSTR(cRec,335,4)
D_DATE     |D|  8|  0| CDATE2DT(SUBSTR(cRec,341,6))
BILL_CLASS |C|  1|  0| SUBSTR(cRec,347,1)
FIRE       |C|  2|  0| SUBSTR(cRec,348,2)
SPEC_DIST1 |C|  3|  0| SUBSTR(cRec,350,3)
SPEC_DIST2 |C|  3|  0| SUBSTR(cRec,353,3)
SPEC_DIST3 |C|  3|  0| SUBSTR(cRec,356,3)
CITY       |C|  3|  0| SUBSTR(cRec,359,3)
TS_PRICE   |N| 11|  0| VAL(SUBSTR(cRec,363,11))
TS_DATE    |D|  8|  0| CDATE2DT(SUBSTR(cRec,376,6))
TS_QFLAG   |C|  1|  0| SUBSTR(cRec,382,1)
L_PRICE    |N| 11|  0| VAL(SUBSTR(cRec,384,11))
L_DATE     |D|  8|  0| CDATE2DT(SUBSTR(cRec,397,6))
L_QFLAG    |C|  1|  0| SUBSTR(cRec,403,1)
PIN_MAP    |C|  5|  0| SUBSTR(cRec,405,5)
PIN_MAPPG  |C|  3|  0| SUBSTR(cRec,411,3)
PIN_BLOCK  |C|  3|  0| SUBSTR(cRec,415,3)
PIN_LOTEXT |C|  7|  0| SUBSTR(cRec,419,7)
CALC_ACRES |N|  8|  2| VALIMPDEC(SUBSTR(cRec,427,7))
VCS        |C|  5|  0| SUBSTR(cRec,439,5)
EQ_ZONING  |C|  5|  0| SUBSTR(cRec,440,5)
ETJ        |C|  2|  0| SUBSTR(cRec,445,2)
C_DATE     |C|  5|  0| SUBSTR(cRec,448,5)
C_LAND_VAL |N| 11|  0| VAL(SUBSTR(cRec,439,11))
C_BLDG_VAL |N| 11|  0| VAL(SUBSTR(cRec,466,11))
C_OBLD_VAL |N|  9|  0| VAL(SUBSTR(cRec,478,9))
C_TBAC_VAL |N|  7|  0| VAL(SUBSTR(cRec,488,7))
LAND_CLASS |C|  1|  0| SUBSTR(cRec,439,1)
LAND_LINE1 |C| 11|  0| SUBSTR(cRec,498,11)
LAND_LINE2 |C| 11|  0| SUBSTR(cRec,510,11)
LAND_LINE3 |C| 11|  0| SUBSTR(cRec,522,11)
LAND_LINE4 |C| 11|  0| SUBSTR(cRec,534,11)
LAND_LINE5 |C| 11|  0| SUBSTR(cRec,546,11)
LAND_LINE6 |C| 11|  0| SUBSTR(cRec,558,10)
USE_FLAG   |C|  6|  0| SUBSTR(cRec,568,1)
